@@ -2,6 +2,7 @@
  * This file is part of Maemo 5 Office UI for KOffice
  *
  * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+ * Copyright (C) 2010 Boudewijn Rempt <boud@kogmbh.com>
  *
  * Contact: Manikandaprasad N C <manikandaprasad.chandrasekar@nokia.com>
  *
@@ -29,29 +30,35 @@
 #include <KoDocument.h>
 #include <KWView.h>
 #include <styles/KoListStyle.h>
+#include <KoPAView.h>
 
-#include "PresentationTool.h"
 #include "Splash.h"
-class KoTextEditor;
+#include "PreviewDialog.h"
+#include "NotesDialog.h"
+#include "SlidingMotionDialog.h"
+
+#include "CollabClient.h"
+#include "CollabDialog.h"
+#include "CollabServer.h"
+#include "FoCellTool.h"
+#include "VirtualKeyBoard.h"
+#include "FoCellToolFactory.h"
+
+
+#ifdef Q_WS_MAEMO_5
+#include "Accelerator.h"
+#endif
+
+
 class QPushButton;
 class QIcon;
 class QTextDocument;
 class QToolButton;
 class QFrame;
 class QLabel;
-class PresentationTool;
-
-namespace Ui
-{
-class MainWindow;
-}
-
 class QMessageBox;
 class QLineEdit;
-class KoCanvasController;
 class QTextCursor;
-class KoShape;
-class KoPAPageBase;
 class QShortcut;
 class QCheckBox;
 class QComboBox;
@@ -60,9 +67,29 @@ class QTextListFormat;
 class QTextDocument;
 class QGridLayout;
 class QDialog;
+class QListWidget;
+class QListWidgetItem;
+class SlidingMotionDialog;
+class GLPresenter;
+class FoCellTooFactory;
+
+class KUndoStack;
+class KoTextEditor;
+class PresentationTool;
+class MainWindowAdaptor;
+class KoCanvasController;
+class KoCanvasControllerWidget;
+class KoShape;
+class KoPAPageBase;
 
 class OfficeInterface;
 
+namespace Ui
+{
+class MainWindow;
+}
+
+
 /*!
  * \brief Main window of the application. KoCanvasController is set as
  * the central widget. It displays the loaded documents.
@@ -74,7 +101,10 @@ class MainWindow : public QMainWindow
 public:
     MainWindow(Splash *aSplash, QWidget *parent = 0);
     ~MainWindow();
-    QString newdocumentype;
+    /*!
+     * Opened file
+     */
+    QString m_fileName;
     //void tabletEvent ( QTabletEvent * event );
     void mousePressEvent ( QMouseEvent * event );
     void mouseMoveEvent ( QMouseEvent * event );
@@ -83,18 +113,164 @@ public:
 
     void disableFullScreenPresentationNavigation();
     void enableFullScreenPresentationNavigation();
+    void openNewDocumentType(QString type);
+
+protected:
+        void closeEvent(QCloseEvent *);
 
 private:
     Ui::MainWindow *m_ui;
-    int m_count;
+    FoCellToolFactory *m_cellToolFactory;
+    
+///////////////////////////
+// Collaborative editing //
+///////////////////////////
+
+// TODO: Better error handling
+
+private:
+    CollabDialog* m_collabDialog;
+    Collaborate* m_collab;
+    KoTextEditor* m_collabEditor;
+
+    void closeCollabDialog();
+
+private slots:
+    void startCollaborating();
+    void collaborationCancelled();
+    void collaborateDialog();
+
+    void receivedFontSize(uint start, uint end, uint size);
+    void receivedTextColor(uint start, uint end, QRgb color);
+    void receivedTextBackgroundColor(uint start, uint end, QRgb color);
+    void receivedFontType(uint start, uint end, const QString &font);
+    void receivedString(uint start, uint end, QByteArray msg);
+    void receivedBackspace(uint start, uint end);
+    void receivedFormat(uint start, uint end, Collaborate::FormatFlag format);
+    void error(quint16 err);
+
+    void collabSaveFile(const QString &filename);
+    void collabOpenFile(const QString &filename);
+
+/////////////////////////////
+/////////////////////////////
+
+
+/////virtualkeyboard
+
+private:
+
+    QShortcut *shortcutForVirtualKeyBoard;
+    QShortcut *spaceHandlerShortcutForVirtualKeyBoard;
+    static bool virtualKeyBoardIsOnScreen;
+
+
+private slots:
+
+    void showVirtualKeyBoardOnScreen();
+    void spaceHandlerForVirtualKeyBoard();
+
+    ////////////////////
+    ////////////////////////////////////
+    // Accelerator scrolling ,sliding //
+    ////////////////////////////////////
+#ifdef Q_WS_MAEMO_5
+
+private:
+
+/*!
+ * true if scrolling start
+ */
+
+static bool stopAcceleratorScrolling;
+/*!
+ * true if slide transition starts
+ */
+static bool enableAccelerator;
+/*!
+ * true if scrolling starts
+ */
+static bool enableScrolling;
+
+
+private:
+/*!
+ *  For accessing the accelerator
+ */
+AcceleratorScrollSlide acceleratorForScrolAndSlide;
+/*!
+ * Pointer to fullscreen Accelerometer Button
+ */
+QPushButton *m_fsAccButton;
+
+/*!
+ * shortcut for enabling the gestures
+ */
+QShortcut *shortcutForAccelerator;
+
+
+
+private slots:
+/*!
+ * Slot to toggle between the slide transition
+ */
+void switchToSlid();
+/*!
+ * Slot to toggle between the scrolling
+ */
+void switchToScroll();
+/*!
+ * Slot for the scroll action
+ */
+void scrollAction();
+/*!
+ * Slot for closing all the accelerator settings
+ */
+void closeAcceleratorSettings();
+/*!
+ * Slot for Enabling and disabling the scroll
+ */
+void  enableDisableScrollingOption();
+
+friend void AcceleratorScrollSlide::ifYesScroll();
+
+friend void AcceleratorScrollSlide::ifNoScroll();
+
+#endif
+////////////////////////////////
+////////////////////////////////
+
+private:
+    /*!
+      * Stores the URL of the open file
+      */
+    KUrl m_url;
+    /*!
+      * Holds the properties of presentation style and time
+      */
+    int gl_showtime;
+    int gl_style;
+    /*!
+     * X-cordinate and Y-cordinate values at mouse click position
+     */
+    int m_xcordinate ;
+    int m_ycordinate ;
+    /*!
+     * Font related information for each Character in NewDocuments 
+     */
+    int m_fontsize;
+    /*!
+     * Dialog for fontsize
+     */
+    QDialog *m_fontSizeDialog;
+    QVBoxLayout *m_fontSizeDialogLayout;
+    QLineEdit *m_fontSizeLineEdit;
+    QListWidget *m_fontSizeList;
     /*!
      * Format frame declaration
      */
     QFrame * m_formatframe;
     QGridLayout * m_formatframelayout;
-    QPushButton * m_bold;
-    QPushButton * m_italic;
-    QPushButton * m_underline;
     QPushButton * m_alignleft;
     QPushButton * m_alignright;
     QPushButton * m_aligncenter;
@@ -107,11 +283,14 @@ private:
     QFrame * m_fontstyleframe;
     QGridLayout *m_fontstyleframelayout;
     QFontComboBox *m_fontcombobox;
-    QComboBox *m_fontsizecombo;
+    QPushButton *m_fontsizebutton;
     QPushButton *m_textcolor;
+    QPushButton *m_textbackgroundcolor;
     QPushButton *m_superscript;
     QPushButton *m_subscript;
-    QPushButton *m_textbackgroundcolor;
+    QPushButton * m_bold;
+    QPushButton * m_italic;
+    QPushButton * m_underline;
     /*!
      * New document chooser dialog
      */
@@ -120,15 +299,47 @@ private:
     QToolButton * m_document;
     QToolButton * m_presenter;
     QToolButton * m_spreadsheet;
+    /*!
+     * Presentation Template chooser dialog
+     */
+    QDialog * m_tempselectiondialog;
+    QGridLayout * m_tempdialoglayout;
+    QListWidget * m_templateWidget;
+    QPushButton * m_go;
+    QPushButton * m_closetemp;
+    QLabel * m_templatepreview;
+    QStringList m_temptitle;
+    QStringList m_templatepath;
+    int m_tempnumber;
+    QString newpresenter;
     /*!
      * Confirmation dialog for closing new document
      */
     QDialog *m_confirmationdialog;
-    QGridLayout * m_confirmationdialoglayout;
+    QGridLayout *m_confirmationdialoglayout;
     QPushButton *m_yes;
     QPushButton *m_no;
     QPushButton *m_cancel;
     QLabel *m_message;
+    /*!
+     * Actions for spreadEditToolBar.
+     */
+    QAction *m_addAction;
+    QAction *m_subtractAction;
+    QAction *m_multiplyAction;
+    QAction *m_divideAction;
+    QAction *m_percentageAction;
+    QAction *m_equalsAction;
+    QSignalMapper *m_signalMapper;
+    /*!
+     * Frame for Spreadsheet sheet information.
+     */
+    QFrame *m_sheetInfoFrame;
+    QGridLayout *m_sheetInfoFrameLayout;
+    QPushButton *m_addSheet;
+    QPushButton *m_removeSheet;
+    QPushButton *m_sheetName;
+
     /*!
      * line edit for search
      */
@@ -160,7 +371,15 @@ private:
     /*!
      * Pointer to KoCanvasController
      */
-    KoCanvasController *m_controller;
+    KoCanvasControllerWidget *m_controller;
+    /*!
+     * Pointer to KUndoStack
+     */
+    KUndoStack *m_undostack;
+    /*!
+     * Pointer to FreOffice CellTool
+     */
+    FoCellTool *m_focelltool;
     /*!
      * Integers about current page
      */
@@ -218,11 +437,6 @@ private:
      */
     enum DocumentType { Text, Presentation, Spreadsheet };
     DocumentType m_type;
-    /*!
-     * Flag to make higlighting of alignment button enable
-     */
-    enum AlignmentType {Left=1,Right,Center,Justify};
-    AlignmentType m_alignType;
     /*!
      * Pointer to splash class
      */
@@ -239,6 +453,10 @@ private:
      * Pointer to presentation drawing tools
      */
     PresentationTool *m_pptTool;
+    /*!
+     *Pointer to the MainWindowAdaptor object
+     */
+    MainWindowAdaptor *m_dbus;
 
     void init();
     /*!
@@ -248,28 +466,36 @@ private:
     /*!
      * style formatting function
      */
-    void doStyle(KoListStyle::Style);
+    void doStyle(KoListStyle::Style, KoTextEditor* editor);
     /*!
      *opening a new document
      */
     void openNewDocument(DocumentType);
+    /*!
+     *Applies appropiate file extension during saving operation
+     */
+    QString getFileExtension(DocumentType);
     /*!
      *Function to add formatframe components
      */
-    QPushButton *addFormatFrameComponent(QString const& imagepath);
+    QPushButton *addFormatFrameComponent(const QString &imagepath);
     /*!
      *Function to add fontstyleframe components
      */
-    QPushButton *addFontStyleFrameComponent(QString const& imagepath);
+    QPushButton *addFontStyleFrameComponent(const QString &imagepath);
     /*!
      *Function to create new document
      */
-    QToolButton *addNewDocument(QString const& docname);
+    QToolButton *addNewDocument(const QString &docname);
     /*!
      * Find string from document
      * /param pointer to QTextDocument
      * /param reference to text to be searched
      */
+    GLPresenter *presenter;
+    /*!
+     * Open GL Class , to handle the slide show.
+     */
     void findText(QList<QTextDocument*> docs, QList<QPair<KoPAPageBase*, KoShape*> > shapes, const QString &aText);
     /*!
      * Find string from document
@@ -314,11 +540,72 @@ private:
      * _NET_ACTIVE_WINDOW message to Matchbox and window is raised.
      */
     void raiseWindow(void);
-
     /*!
-     * counts initial undosteps of KoDocument
+     * Function to check activeFormatFrame Options
      */
-    void initialUndoStepsCount();
+    void activeFormatOptionCheck();
+    /*!
+     * Function to check activeFontStyleFrame Options
+     */
+    void activeFontOptionCheck();
+    /*!
+     * Function for navigating to NextSheet
+     */
+    void nextSheet();
+    /*!
+     * Function for navigating to PreviousSheet
+     */
+    void prevSheet();
+    /*!
+     * Confirmation Dialog Destructor
+     */
+    void confirmationDialogDestructor();
+    /*!
+     * Format Frame Destructor
+     */
+    void formatFrameDestructor();
+    /*!
+     * FontStyle Frame Destructor
+     */
+    void fontStyleFrameDestructor();
+    /*!
+     * Template Chooser
+     */
+    void templateSelectionDialog();
+    /*!
+     *
+     */
+    bool m_firstChar;
+    /*!
+     * Set up tool bar.
+     */
+    void setUpSpreadEditorToolBar();
+    /*!
+     * Reset the SpreadEditor toolbar.
+     */
+    void resetSpreadEditorToolBar();
+
+private:
+    // Apply the selected formatting
+    bool setFontSize(int size, KoTextEditor* editor);
+    bool setFontType(const QString &font, KoTextEditor* editor);
+    bool setTextColor(const QColor &color, KoTextEditor* editor);
+    bool setTextBackgroundColor(const QColor &color, KoTextEditor* editor);
+    bool setBold(KoTextEditor* editor);
+    bool setItalic(KoTextEditor* editor);
+    bool setUnderline(KoTextEditor* editor);
+    bool setLeftAlign(KoTextEditor* editor);
+    bool setRightAlign(KoTextEditor* editor);
+    bool setCenterAlign(KoTextEditor* editor);
+    bool setJustify(KoTextEditor* editor);
+    bool setNumberList(KoTextEditor* editor);
+    bool setBulletList(KoTextEditor* editor);
+    bool setSubScript(KoTextEditor* editor);
+    bool setSuperScript(KoTextEditor* editor);
+    /*!
+     * Handle the Spreadsheet sheet information
+     */
+    void spreadSheetInfo();
 
 private slots:
 
@@ -326,6 +613,26 @@ private slots:
     void pluginOpen(bool newWindow, const QString& path);
     void updateUI();
     void resourceChanged(int key, const QVariant &value);
+    void showFontSizeDialog();
+    void fontSizeEntered();
+    void fontSizeRowSelected(QListWidgetItem *item);
+    void startMathMode(bool start);
+    /**
+     * Adds symbol for spreadEditToolBar
+     */
+    void addMathematicalOperator(QString mathSymbol);
+    /*!
+     * Remove a sheet.
+     */
+    void removeSheet();
+    /*!
+     * Add add a sheet.
+     */
+    void addSheet();
+    /*!
+     * Current sheet info.
+     */
+    QString currentSheetName();
     /*!
      * Slot to perform UndoAction
      */
@@ -393,7 +700,7 @@ private slots:
     /*!
      *  Slot for font size Selection
      */
-    void selectFontSize();
+    void selectFontSize(int size);
     /*!
      *  Slot for font type Selection
      */
@@ -422,18 +729,10 @@ private slots:
      * Slot to display fontstyleframe with all options
      */
      void openFontStyleFrame();
-    /*!
-     * Slot to choose new document
-     */
-    void chooseDocumentType();
-    /*!
-     * Slot to find AlignmentType
-     */
-    void findAlignType();
-    /*!
-     * Slot for intial AlignmentTypeCheck
-     */
-    void initialAlignType();
+     /*!
+      * Slot to display Math operation options
+      */
+    void openMathOpFrame();
     /*!
      * Slot to open new documnet
      */
@@ -518,7 +817,7 @@ private slots:
     /*!
      * Slot that is invoked when the currently active tool changes.
      */
-    void activeToolChanged(KoCanvasController *canvas, int uniqueToolId);
+    void activeToolChanged(KoCanvasControllerWidget *canvas, int uniqueToolId);
     /*!
      * function for opening existing document.
      */
@@ -527,10 +826,28 @@ private slots:
      * Slot to actionAbout triggered signal
      */
     void openAboutDialog();
+    /*!
+     * Slot to show  slide transition options
+     */
+    void slideTransitionDialog();
+    /*!
+     * Slot to preview selected templates
+     */
+    void selectedTemplatePreview(int number);
+    /*!
+     * Open selected template
+     */
+    void openSelectedTemplate();
+    /*!
+     * Close Template Selection Dialog
+     */
+    void closeTempSelectionDialog();
     /*!
      * Slot for progress indicator
      */
     //void slotProgress(int value);
+
+    void inserNewSlide();
 public slots:
     /*!
      * Slot to perform save operation
@@ -545,6 +862,10 @@ public slots:
      * /param filename
      */
     void openDocument(const QString &fileName);
+    /*!
+     * Slot to choose new document
+     */
+    void chooseDocumentType();
     /*!
      * Slot to actionOpen triggered signal
      */
@@ -561,40 +882,62 @@ public slots:
      * Then it quits.
      */
     void loadScrollAndQuit();
+    /*!
+     * Slot to go to perticular page
+     */
+    void gotoPage(int page);
+    /*!
+     * Slot to show the preview dialog
+     */
+    void showPreviewDialog();
+    /*!
+     * Slot to show the notes dialog when show notes dialog is clicked
+     */
+    void slideNotesButtonClicked();
+    /*!
+     * Slot to update current slide in the presentation, when the slide changed in the notes dialog
+     */
+    void moveSLideFromNotesSLide(bool flag);
 
+    void glPresenter();
+    void glPresenterSet(int,int);
 private:
 
     QMap<QString, OfficeInterface*> loadedPlugins;
 
     void setShowProgressIndicator(bool visible);
-    /*!
-     * Opened file
-     */
-    QString m_fileName;
-    /*!
-     * Double click detector
-     */
-    bool m_doubleClick;
-    /*!
-     * true if existing document is open
-     */
-    bool m_openCheck;
     /*!
      * true if new document is open
      */
     bool m_newDocOpen;
+    /*!
+     * true if document is modified
+     */
+    bool m_isDocModified;
     /*!
      * flag for new file to existing file conversion
      */
     bool m_existingFile;
     /*!
-     * true if already one document exists
+     * QShortcut to cut text with Ctrl-X
      */
-    bool m_docExist;
+    QShortcut *m_cutShortcut;
     /*!
-     * QShortcut for copying text with Ctrl-c
+     * QShortcut for copying text with Ctrl-C
      */
     QShortcut *m_copyShortcut;
+    /*!
+     * QShortcut for copying text with Ctrl-V
+     */
+    QShortcut *m_pasteShortcut;
+    /*!
+     * QShortcut for copying text with Ctrl-Z
+     */
+    QShortcut *m_undoShortcut;
+    /*!
+     * QShortcut for copying text with Ctrl-P
+     */
+    QShortcut *m_redoShortcut;
     /*!
      * Count of mouseMove or tabletMove events after MousePress or tabletPress event
      */
@@ -611,6 +954,52 @@ private:
      * Is document is currently being loaded
      */
     bool m_isLoading;
+    /*!
+     * pointer to preview button store
+     */
+    StoreButtonPreview *storeButtonPreview;
+    /*!
+     * view number used while dbus session creation
+     */
+    int viewNumber;
+    /*!
+     * Pointer to show notes button
+     */
+    QPushButton *m_slideNotesButton;
+    /*!
+     * Icon for show notes button
+     */
+    const QIcon m_slideNotesIcon;
+    /*!
+     * Pointer to notes dialog
+     */
+    NotesDialog *notesDialog;
+    /*!
+     * Pointer to sliding motion dialog
+     */
+    SlidingMotionDialog *m_slidingmotiondialog;
+
+    /*!
+     * Tool bar for spread sheet.
+     */
+    QToolBar *m_spreadEditToolBar;
+signals:
+    /*!
+     * Presentation has entered full screen mode.
+     */
+    void presentationStarted();
+    /*!
+     * Presentation has exited from full screen mode.
+     */
+    void presentationStopped();
+    /*!
+     * Presentation has moved to the next slide.
+     */
+    void nextSlide();
+    /*!
+     * Presentation has moved to the previous slide.
+     */
+    void previousSlide();
 };
 
 #endif // MAINWINDOW_H
